@@ -1,8 +1,8 @@
 """Provides Authentication and Authorization classes."""
-import aiohttp
 import inspect
 import time
 
+import aiohttp
 from aiohttp import ClientRequest
 from yarl import URL
 
@@ -19,10 +19,10 @@ class BaseAuthenticator(object):
 
         :param requestor: An instance of :class:`Requestor`.
         :param client_id: The OAuth2 client ID to use with the session.
-        :param redirect_uri: (optional) The redirect URI exactly as specified
-            in your OAuth application settings on Reddit. This parameter is
-            required if you want to use the ``authorize_url`` method, or the
-            ``authorize`` method of the ``Authorizer`` class.
+        :param redirect_uri: (optional) The redirect URI exactly as specified in your
+            OAuth application settings on Reddit. This parameter is required if you want
+            to use the ``authorize_url`` method, or the ``authorize`` method of the
+            ``Authorizer`` class.
 
         """
         self._requestor = requestor
@@ -45,18 +45,17 @@ class BaseAuthenticator(object):
         """Return the URL used out-of-band to grant access to your application.
 
         :param duration: Either ``permanent`` or ``temporary``. ``temporary``
-            authorizations generate access tokens that last only 1
-            hour. ``permanent`` authorizations additionally generate a refresh
-            token that can be indefinitely used to generate new hour-long
-            access tokens. Only ``temporary`` can be specified if ``implicit``
-            is set to ``True``.
+            authorizations generate access tokens that last only 1 hour. ``permanent``
+            authorizations additionally generate a refresh token that can be
+            indefinitely used to generate new hour-long access tokens. Only
+            ``temporary`` can be specified if ``implicit`` is set to ``True``.
         :param scopes: A list of OAuth scopes to request authorization for.
         :param state: A string that will be reflected in the callback to
             ``redirect_uri``. Elements must be printable ASCII characters in the range
             0x20 through 0x7E inclusive. This value should be temporarily unique to the
             client for whom the URL was generated.
-        :param implicit: (optional) Use the implicit grant flow (default:
-            False). This flow is only available for UntrustedAuthenticators.
+        :param implicit: (optional) Use the implicit grant flow (default: False). This
+            flow is only available for UntrustedAuthenticators.
 
         """
         if self.redirect_uri is None:
@@ -87,9 +86,9 @@ class BaseAuthenticator(object):
         """Ask Reddit to revoke the provided token.
 
         :param token: The access or refresh token to revoke.
-        :param token_type: (Optional) When provided, hint to Reddit what the
-            token type is for a possible efficiency gain. The value can be
-            either ``access_token`` or ``refresh_token``.
+        :param token_type: (Optional) When provided, hint to Reddit what the token type
+            is for a possible efficiency gain. The value can be either ``access_token``
+            or ``refresh_token``.
 
         """
         data = {"token": token}
@@ -110,10 +109,10 @@ class TrustedAuthenticator(BaseAuthenticator):
         :param requestor: An instance of :class:`Requestor`.
         :param client_id: The OAuth2 client ID to use with the session.
         :param client_secret: The OAuth2 client secret to use with the session.
-        :param redirect_uri: (optional) The redirect URI exactly as specified
-            in your OAuth application settings on Reddit. This parameter is
-            required if you want to use the ``authorize_url`` method, or the
-            ``authorize`` method of the ``Authorizer`` class.
+        :param redirect_uri: (optional) The redirect URI exactly as specified in your
+            OAuth application settings on Reddit. This parameter is required if you want
+            to use the ``authorize_url`` method, or the ``authorize`` method of the
+            ``Authorizer`` class.
 
         """
         super(TrustedAuthenticator, self).__init__(requestor, client_id, redirect_uri)
@@ -173,8 +172,8 @@ class BaseAuthorizer(object):
     def is_valid(self):
         """Return whether or not the Authorizer is ready to authorize requests.
 
-        A ``True`` return value does not guarantee that the access_token is
-        actually valid on the server side.
+        A ``True`` return value does not guarantee that the access_token is actually
+        valid on the server side.
 
         """
         return (
@@ -205,8 +204,7 @@ class Authorizer(BaseAuthorizer):
     ):
         """Represent a single authorization to Reddit's API.
 
-        :param authenticator: An instance of a subclass of
-            :class:`BaseAuthenticator`.
+        :param authenticator: An instance of a subclass of :class:`BaseAuthenticator`.
         :param post_refresh_callback: (Optional) When a single-argument synchronous or
             asynchronous function is passed, the function will be called prior to
             refreshing the access and refresh tokens. The argument to the callback is
@@ -229,8 +227,8 @@ class Authorizer(BaseAuthorizer):
     async def authorize(self, code):
         """Obtain and set authorization tokens based on ``code``.
 
-        :param code: The code obtained by an out-of-band authorization request
-            to Reddit.
+        :param code: The code obtained by an out-of-band authorization request to
+            Reddit.
 
         """
         if self._authenticator.redirect_uri is None:
@@ -262,11 +260,11 @@ class Authorizer(BaseAuthorizer):
     async def revoke(self, only_access=False):
         """Revoke the current Authorization.
 
-        :param only_access: (Optional) When explicitly set to True, do not
-            evict the refresh token if one is set.
+        :param only_access: (Optional) When explicitly set to True, do not evict the
+            refresh token if one is set.
 
-        Revoking a refresh token will in-turn revoke all access tokens
-        associated with that authorization.
+        Revoking a refresh token will in-turn revoke all access tokens associated with
+        that authorization.
 
         """
         if only_access or self.refresh_token is None:
@@ -280,8 +278,8 @@ class Authorizer(BaseAuthorizer):
 class DeviceIDAuthorizer(BaseAuthorizer):
     """Manages app-only OAuth2 for 'installed' applications.
 
-    While the '*' scope will be available, some endpoints simply will not work
-    due to the lack of an associated Reddit account.
+    While the '*' scope will be available, some endpoints simply will not work due to
+    the lack of an associated Reddit account.
 
     """
 
@@ -291,10 +289,10 @@ class DeviceIDAuthorizer(BaseAuthorizer):
         """Represent an app-only OAuth2 authorization for 'installed' apps.
 
         :param authenticator: An instance of :class:`UntrustedAuthenticator`.
-        :param device_id: (optional) A unique ID (20-30 character ASCII string)
-            (default DO_NOT_TRACK_THIS_DEVICE). For more information about this
-            parameter, see:
+        :param device_id: (optional) A unique ID (20-30 character ASCII string) (default
+            DO_NOT_TRACK_THIS_DEVICE). For more information about this parameter, see:
             https://github.com/reddit/reddit/wiki/OAuth2#application-only-oauth
+
         """
         super(DeviceIDAuthorizer, self).__init__(authenticator)
         self._device_id = device_id
@@ -314,17 +312,15 @@ class ImplicitAuthorizer(BaseAuthorizer):
         """Represent a single implicit authorization to Reddit's API.
 
         :param authenticator: An instance of :class:`UntrustedAuthenticator`.
-        :param access_token: The access_token obtained from Reddit via callback
-            to the authenticator's redirect_uri.
-        :param expires_in: The number of seconds the ``access_token`` is valid
-            for. The origin of this value was returned from Reddit via callback
-            to the authenticator's redirect uri. Note, you may need to subtract
-            an offset before passing in this number to account for a delay
-            between when Reddit prepared the response, and when you make this
-            function call.
-        :param scope: A space-delimited string of Reddit OAuth2 scope names as
-            returned from Reddit in the callback to the authenticator's
-            redirect uri.
+        :param access_token: The access_token obtained from Reddit via callback to the
+            authenticator's redirect_uri.
+        :param expires_in: The number of seconds the ``access_token`` is valid for. The
+            origin of this value was returned from Reddit via callback to the
+            authenticator's redirect uri. Note, you may need to subtract an offset
+            before passing in this number to account for a delay between when Reddit
+            prepared the response, and when you make this function call.
+        :param scope: A space-delimited string of Reddit OAuth2 scope names as returned
+            from Reddit in the callback to the authenticator's redirect uri.
 
         """
         super(ImplicitAuthorizer, self).__init__(authenticator)
@@ -336,8 +332,8 @@ class ImplicitAuthorizer(BaseAuthorizer):
 class ReadOnlyAuthorizer(Authorizer):
     """Manages authorizations that are not associated with a Reddit account.
 
-    While the '*' scope will be available, some endpoints simply will not work
-    due to the lack of an associated Reddit account.
+    While the '*' scope will be available, some endpoints simply will not work due to
+    the lack of an associated Reddit account.
 
     """
 
@@ -358,22 +354,32 @@ class ScriptAuthorizer(Authorizer):
 
     AUTHENTICATOR_CLASS = TrustedAuthenticator
 
-    def __init__(self, authenticator, username, password):
+    def __init__(self, authenticator, username, password, two_factor_callback=None):
         """Represent a single personal-use authorization to Reddit's API.
 
         :param authenticator: An instance of :class:`TrustedAuthenticator`.
         :param username: The Reddit username of one of the application's developers.
         :param password: The password associated with ``username``.
+        :param two_factor_callback: A synchronous or asynchronous function that returns
+            OTPs (One-Time Passcodes), also known as 2FA auth codes. If this function is
+            provided, prawcore will call it when authenticating.
 
         """
         super(ScriptAuthorizer, self).__init__(authenticator)
         self._username = username
         self._password = password
+        self._two_factor_callback = two_factor_callback
 
     async def refresh(self):
         """Obtain a new personal-use script type access token."""
+        if self._two_factor_callback:
+            if inspect.iscoroutinefunction(self._two_factor_callback):
+                otp = await self._two_factor_callback()
+            else:
+                otp = self._two_factor_callback()
         await self._request_token(
             grant_type="password",
             username=self._username,
             password=self._password,
+            otp=self._two_factor_callback and otp,
         )
