@@ -4,11 +4,11 @@ import json
 import os
 from base64 import b64encode
 from datetime import datetime
-from vcr import VCR
+
+from vcr import VCR as _VCR
 from vcr.persisters.filesystem import FilesystemPersister
 from vcr.serialize import deserialize, serialize
 
-
 CLIENT_ID = os.environ.get("PRAWCORE_CLIENT_ID", "fake_client_id")
 CLIENT_SECRET = os.environ.get("PRAWCORE_CLIENT_SECRET", "fake_client_secret")
 PASSWORD = os.environ.get("PRAWCORE_PASSWORD", "fake_password")
@@ -24,6 +24,11 @@ def b64_string(input_string):
     return b64encode(input_string.encode("utf-8")).decode("utf-8")
 
 
+def two_factor_callback():
+    """Return an OTP code."""
+    return None
+
+
 placeholders = [
     ("<CLIENT_ID>", CLIENT_ID),
     ("<CLIENT_SECRET>", CLIENT_SECRET),
@@ -127,7 +132,7 @@ class CustomPersister(FilesystemPersister):
             f.write(data)
 
 
-class CustomVCR(VCR):
+class CustomVCR(_VCR):
     """Derived from VCR to make setting paths easier."""
 
     def use_cassette(self, path="", **kwargs):
