@@ -1,14 +1,15 @@
 """Test for sublcasses of asyncprawcore.auth.BaseAuthenticator class."""
-import asyncprawcore
 import asynctest
 
-from .conftest import CLIENT_ID, CLIENT_SECRET, REDIRECT_URI, VCR
+import asyncprawcore
 from asyncprawcore.requestor import Requestor
 
+from .conftest import CLIENT_ID, CLIENT_SECRET, REDIRECT_URI, VCR
+
 
 class TrustedAuthenticatorTest(asynctest.TestCase):
     async def setUp(self) -> None:
-        self.requestor = Requestor("asyncprawcore:test (by /u/bboe)")
+        self.requestor = Requestor("asyncprawcore:test (by /u/Lil_SpazJoekp)")
 
     async def tearDown(self) -> None:
         await self.requestor.close()
@@ -77,7 +78,7 @@ class TrustedAuthenticatorTest(asynctest.TestCase):
 
 class UntrustedAuthenticatorTest(asynctest.TestCase):
     async def setUp(self) -> None:
-        self.requestor = Requestor("asyncprawcore:test (by /u/bboe)")
+        self.requestor = Requestor("asyncprawcore:test (by /u/Lil_SpazJoekp)")
 
     async def tearDown(self) -> None:
         await self.requestor.close()
