@@ -10,20 +10,24 @@ from testfixtures import LogCapture
 
 import asyncprawcore
 from asyncprawcore.exceptions import RequestException
+
 from .conftest import (
-    AsyncMock,
     CLIENT_ID,
     CLIENT_SECRET,
     PASSWORD,
     REFRESH_TOKEN,
     USERNAME,
     VCR,
+    AsyncMock,
+    two_factor_callback,
 )
 
 
 class InvalidAuthorizer(asyncprawcore.Authorizer):
     def __init__(self):
-        requestor = asyncprawcore.requestor.Requestor("asyncprawcore:test (by /u/bboe)")
+        requestor = asyncprawcore.requestor.Requestor(
+            "asyncprawcore:test (by /u/Lil_SpazJoekp)"
+        )
 
         super(InvalidAuthorizer, self).__init__(
             asyncprawcore.TrustedAuthenticator(requestor, CLIENT_ID, CLIENT_SECRET)
@@ -34,7 +38,9 @@ class InvalidAuthorizer(asyncprawcore.Authorizer):
 
 
 async def client_authorizer():
-    requestor = asyncprawcore.requestor.Requestor("asyncprawcore:test (by /u/bboe)")
+    requestor = asyncprawcore.requestor.Requestor(
+        "asyncprawcore:test (by /u/Lil_SpazJoekp)"
+    )
 
     authenticator = asyncprawcore.TrustedAuthenticator(
         requestor, CLIENT_ID, CLIENT_SECRET
@@ -44,8 +50,10 @@ async def client_authorizer():
     return authorizer
 
 
-async def readonly_authorizer(refresh=True):
-    requestor = asyncprawcore.requestor.Requestor("asyncprawcore:test (by /u/bboe)")
+async def readonly_authorizer(refresh=True, requestor=None):
+    requestor = requestor or asyncprawcore.requestor.Requestor(
+        "asyncprawcore:test (by /u/Lil_SpazJoekp)"
+    )
     authenticator = asyncprawcore.TrustedAuthenticator(
         requestor, CLIENT_ID, CLIENT_SECRET
     )
@@ -56,11 +64,15 @@ async def readonly_authorizer(refresh=True):
 
 
 async def script_authorizer():
-    requestor = asyncprawcore.requestor.Requestor("asyncprawcore:test (by /u/bboe)")
+    requestor = asyncprawcore.requestor.Requestor(
+        "asyncprawcore:test (by /u/Lil_SpazJoekp)"
+    )
     authenticator = asyncprawcore.TrustedAuthenticator(
         requestor, CLIENT_ID, CLIENT_SECRET
     )
-    authorizer = asyncprawcore.ScriptAuthorizer(authenticator, USERNAME, PASSWORD)
+    authorizer = asyncprawcore.ScriptAuthorizer(
+        authenticator, USERNAME, PASSWORD, two_factor_callback
+    )
     await authorizer.refresh()
     return authorizer
 
@@ -68,7 +80,7 @@ async def script_authorizer():
 class SessionTest(asynctest.TestCase):
     async def setUp(self) -> None:
         self.requestor = asyncprawcore.requestor.Requestor(
-            "asyncprawcore:test (by /u/bboe)"
+            "asyncprawcore:test (by /u/Lil_SpazJoekp)"
         )
 
     async def tearDown(self) -> None:
@@ -374,6 +386,55 @@ class SessionTest(asynctest.TestCase):
                 )
             self.assertEqual(413, context_manager.exception.response.status)
 
+    async def test_request__too__many_requests__with_retry_headers(self):
+        with VCR.use_cassette(
+            "Session_request__too__many_requests__with_retry_headers"
+        ):
+            session = asyncprawcore.Session(await readonly_authorizer())
+            session._requestor._http.headers.update(
+                {"User-Agent": "python-requests/2.25.1"}
+            )
+            with self.assertRaises(asyncprawcore.TooManyRequests) as context_manager:
+                await session.request("GET", "/api/v1/me")
+            self.assertEqual(429, context_manager.exception.response.status)
+            self.assertTrue(
+                context_manager.exception.response.headers.get("retry-after")
+            )
+            self.assertEqual(
+                "Too Many Requests", context_manager.exception.response.reason
+            )
+            self.assertTrue(
+                str(context_manager.exception).startswith(
+                    "received 429 HTTP response. Please wait at least"
+                )
+            )
+            self.assertTrue(
+                (await context_manager.exception.message()).startswith(
+                    "\n<!doctype html>"
+                )
+            )
+
+    async def test_request__too__many_requests__without_retry_headers(self):
+        with VCR.use_cassette(
+            "Session_request__too__many_requests__without_retry_headers"
+        ):
+            requestor = asyncprawcore.Requestor("python-requests/2.25.1")
+            with self.assertRaises(
+                asyncprawcore.exceptions.ResponseException
+            ) as context_manager:
+                asyncprawcore.Session(await readonly_authorizer(requestor=requestor))
+            self.assertEqual(429, context_manager.exception.response.status)
+            self.assertFalse(
+                context_manager.exception.response.headers.get("retry-after")
+            )
+            self.assertEqual(
+                "Too Many Requests", context_manager.exception.response.reason
+            )
+            self.assertEqual(
+                await context_manager.exception.response.json(),
+                {"message": "Too Many Requests", "error": 429},
+            )
+
     async def test_request__unavailable_for_legal_reasons(self):
         with VCR.use_cassette("Session_request__unavailable_for_legal_reasons"):
             authenticator = asyncprawcore.UntrustedAuthenticator(
@@ -401,6 +462,16 @@ class SessionTest(asynctest.TestCase):
                 await session.request("POST", "r/asyncpraw/api/wiki/edit/", data=data)
             self.assertEqual(415, context_manager.exception.response.status)
 
+    async def test_request__uri_too_long(self):
+        with VCR.use_cassette("Session_request__uri_too_long"):
+            self.session = asyncprawcore.Session(await readonly_authorizer())
+            path_start = "/api/morechildren?link_id=t3_n7r3uz&children="
+            with open("tests/files/comment_ids.txt") as fp:
+                ids = fp.read()
+            with self.assertRaises(asyncprawcore.URITooLong) as context_manager:
+                await self.session.request("GET", (path_start + ids)[:9996])
+            self.assertEqual(414, context_manager.exception.response.status)
+
     async def test_request__with_insufficient_scope(self):
         with VCR.use_cassette("Session_request__with_insufficient_scope"):
             self.session = asyncprawcore.Session(await client_authorizer())
